//
//  led7seg.h
//
//  7-Sengent LED Control
//  Copyright (C) 2009-2011 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		7-Segment LEDs display control subroutine.
//		Supported any digits anode/cathode common type.
//		Any signal porality.
//		Segment signal connection is fixed. a=b0= b=b1 c=b3 ... g=b6 dot=b7
//
//  Device: PIC16F88         IDE: MPLAB IDE 8.36, HI-TECH C Tools for the PIC10/12/16 MCU Family Version 9.60PL5
//  Device: PIC18F14K50      IDE: MPLAB IDE 8.36, C Compiler for PIC18 MCUs v3.33
//  Device: PIC18F26J50      IDE: MPLAB IDE 8.36, C Compiler for PIC18 MCUs v3.33
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2009.04.23 v0.0  1st rel(BDCM version,PIC16F88 only)
//    2011.01.01 v0.1  Support PIC18/C18
//    2011.01.25 v0.2  Support multi digit ports & any order
//


#ifndef LED7SEG_H
#define LED7SEG_H

//
//	Hardware
//
// ------------------------- PIC18F26J50 -------------------------
#include "p18cxxx.h"
#define LED7SEG_INIT()		{                                \
	TRISB = ~0b11111111;	/* Set output to segment port */ \
	TRISA &= ~0b01000000;	/* Set output to digit port1  */ \
	TRISC &= ~0b11000100;	/* Set output to digit port2  */ \
}
#define LED7SEG_TERM()		{                               \
	TRISB = 0b11111111;		/* Set input to segment port */ \
	TRISA |= 0b01000000;	/* Set input to digit port1  */ \
	TRISC |= 0b11000100;	/* Set input to digit port2  */ \
}
//	LATB = ~0b11111111;		/* Set low to segment port */
//	LATA &= ~0b01000000;	/* Set low to digit port1  */
//	LATC &= ~0b11000100;	/* Set low to digit port2  */
//	LATB = ~0b11111111;		/* Set high to segment port */
//	LATA &= ~0b01000000;	/* Set high to digit port1  */
//	LATC &= ~0b11000100;	/* Set high to digit port2  */

//#define LED7SEG_SEG_SET(p)	(LATB = (p))									// Active high
//#define LED7SEG_SEG_ALLOFF()	(LATB = 0)										// Active high
#define LED7SEG_SEG_SET(p)		(LATB = ~(p))									// Acvive low
#define LED7SEG_SEG_ALLOFF()	(LATB = ~0)										// Active low
#define LED7SEG_DIG_ON(n)		*led7seg_digtbl_pa[n] |= led7seg_digtbl_pb[n]	// Active high
#define LED7SEG_DIG_OFF(n)		*led7seg_digtbl_pa[n] &= ~led7seg_digtbl_pb[n]	// Active high
#define LED7SEG_DIG_ALLOFF()	{LATA &= ~0b01000000; LATC &= ~0b11000100;}		// Active high
//#define LED7SEG_DIG_ON(p)		*led7seg_digtbl_pa[n] &= ~led7seg_digtbl_pb[n]	// Active low
//#define LED7SEG_DIG_OFF(p)	*led7seg_digtbl_pa[n] |= led7seg_digtbl_pb[n]	// Active low
//#define LED7SEG_DIG_ALLOFF()	{LATA |= 0b01000000; LATC |= 0b11000100;}		// Active low
#define LED7SEG_TBLDEF			unsigned char	*led7seg_digtbl_pa[] = {&LATA, &LATC, &LATC, &LATC};\
								unsigned char	led7seg_digtbl_pb[] = {0b01000000, 0b00000100, 0b01000000, 0b10000000}
extern unsigned char	*led7seg_digtbl_pa[];
extern unsigned char	led7seg_digtbl_pb[];

// ------------------------- PIC18F14K50 -------------------------
//#include "p18cxxx.h"
//#define LED7SEG_INIT()		{                                \
//	TRISC = 0b00000000;		/* Set output to segment port */ \
//	TRISB &= ~0b11110000;	/* Set output to digit port   */ \
//}
//#define LED7SEG_TERM()		{                               \
//	TRISC = 0b11111111;		/* Set input to segment port */ \
//	TRISB |= 0b11110000;	/* Set input to digit port   */ \
//}
////#define LED7SEG_SEG_SET(p)	(LATC = (p))									// Active high
////#define LED7SEG_SEG_ALLOFF()	(LATC = 0)										// Active high
//#define LED7SEG_SEG_SET(p)		(LATC = ~(p))									// Acvive low
//#define LED7SEG_SEG_ALLOFF()	(LATC = ~0)										// Active low
//#define LED7SEG_DIG_ON(n)		*led7seg_digtbl_pa[n] |= led7seg_digtbl_pb[n]	// Active high
//#define LED7SEG_DIG_OFF(n)		*led7seg_digtbl_pa[n] &= ~led7seg_digtbl_pb[n]	// Active high
//#define LED7SEG_DIG_ALLOFF()	(LATB &= ~0b11110000)							// Active high
////#define LED7SEG_DIG_ON(p)		*led7seg_digtbl_pa[n] &= ~led7seg_digtbl_pb[n]	// Active low
////#define LED7SEG_DIG_OFF(p)	*led7seg_digtbl_pa[n] |= led7seg_digtbl_pb[n]	// Active low
////#define LED7SEG_DIG_ALLOFF()	(LATB |= 0b11110000)							// Active low
//#define LED7SEG_TBLDEF			unsigned char	*led7seg_digtbl_pa[] = {&LATB, &LATB, &LATB, &LATB};\
//								unsigned char	led7seg_digtbl_pb[] = {0b00010000, 0b00100000, 0b01000000, 0b10000000}
//extern unsigned char	*led7seg_digtbl_pa[];
//extern unsigned char	led7seg_digtbl_pb[];


//
//	Configure
//
#define LED7SEG_DIG_WIDTH	4				// Physical column(1~)
#define LED7SEG_INIT_PATT	0b00110110		// Initial display pattern


//
//	API
//
extern void led7seg_Start(void);
extern void led7seg_PutPattern(unsigned char pat, unsigned char pos);
//     void led7seg_PutHex(unsigned char hex, unsigned char pos);
#define     led7seg_PutHex(hex, pos)	led7seg_PutPattern((led7seg_Pat[hex]), (pos))
extern void led7seg_DP(unsigned char f, unsigned char pos);
extern void led7seg_Dim(signed char dim);
extern void led7seg_DispInt(int iValue, unsigned char bPos, unsigned char bLen);
extern void led7seg_Refresh(void);		// Need interval call by user (~4[ms])
//     void led7seg_Update(void);
#define     led7seg_Update()			led7seg_Refresh()
extern void led7seg_Stop(void);
// call sequense: led7seg_Start() -> ... -> led7seg_Stop()
// pat:   Segment bit image(b7=dp b6=g b5=f ... b0=a)
// pos:   Digit position(1~LED7SEG_DIG_WIDTH, 0=all digit)
// hex:   Nibble data
// f:     Dot flag(on=1 off=0)
// dim:   Dimmer (bright=1 dark=0 little-dark=-1 more-dark=-2...-8)
// iValue:Display number
// bPos:  Start digit position(1~LED7SEG_DIG_WIDTH)
// bLen:  Display digit width

// Internal table
extern const  unsigned char led7seg_Pat[];

#endif // LED7SEG_H
